//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import Foundation
import RxSwift
import Alamofire
import RxAlamofire
import KakaoPartnerSDKCommon
import KakaoSDKCommon
import RxKakaoSDKCommon
import KakaoSDKAuth
import RxKakaoSDKAuth
import KakaoSDKUser
import RxKakaoSDKUser

import KakaoPartnerSDKUser

let auth = Auth.shared
let api = Api.shared

/// 사용자관리 API 호출을 담당하는 클래스입니다.
extension Reactive where Base: UserApi {
    
    /// 사용자에 대한 다양한 정보를 얻을 수 있습니다.
    /// - seealso: `PartnerUser`
    public func meForPartner(propertyKeys: [String]? = nil) -> Single<PartnerUser> {
        return auth.rx.responseData(.get, Urls.compose(path:Paths.userMe), parameters: ["propertyKeys": propertyKeys].filterNil())
            .compose(auth.rx.checkErrorAndRetryComposeTransformer())
            .map({ (response, data) -> (SdkJSONDecoder, HTTPURLResponse, Data) in
                return (SdkJSONDecoder.customIso8601Date, response, data)
            })
            .compose(api.rx.decodeDataComposeTransformer())
            .asSingle()
    }
    
    /// 앱 연결 상태가 **PREREGISTER** 상태의 사용자에 대하여 앱 연결 요청을 합니다. **자동연결** 설정을 비활성화한 앱에서 사용합니다. 요청에 성공하면 사용자 아이디가 반환됩니다.
    public func signup() -> Single<Int64?> {
        return auth.rx.responseData(.post, Urls.compose(path:PartnerPaths.signup))
            .compose(auth.rx.checkErrorAndRetryComposeTransformer())
            .map({ (response, data) -> Int64? in
                if let json = (try? JSONSerialization.jsonObject(with:data, options:[])) as? [String: Any] {
                    return json["id"] as? Int64
                }
                else {
                    return nil
                }
            })
            .do (
                onNext: { ( decoded ) in
                    SdkLog.i("decoded model:\n \(String(describing: decoded))\n\n" )
                }
            )
            .asSingle()
    }
}
